@@ -80,7 +80,7 @@ void mpd_connect(MpdObj* mo)
    struct hostent* remote_he;
    struct sockaddr* remote_sa;
    struct sockaddr_in remote_si;
-   int flags,err,nbread;
+   int err,nbread;
    struct timeval tv;
    fd_set fds;
 
@@ -112,9 +112,7 @@ void mpd_connect(MpdObj* mo)
       return;
    }
 
-   flags = fcntl(mo->socket, F_GETFL, 0);
-   fcntl(mo->socket, F_SETFL, flags | O_NONBLOCK);
-   if (connect(mo->socket,remote_sa, sizeof(struct sockaddr_in)) < 0 && errno != EINPROGRESS)
+   if (connect(mo->socket,remote_sa, sizeof(struct sockaddr_in)) < 0)
    {
       mo->error = MPD_ERROR_CONNPORT;
       DBG("ERROR @connect(), err=%s",strerror(errno));
@@ -127,7 +125,7 @@ void mpd_connect(MpdObj* mo)
    FD_SET(mo->socket,&fds);
    if((err = select(mo->socket+1,&fds,NULL,NULL,&tv)) == 1)
    {
-      if ((nbread = recv(mo->socket, mo->buffer, MAXBUFLEN, 0)) < 0)
+      if ((nbread = recv(mo->socket, mo->recv_buffer, MAXBUFLEN, 0)) < 0)
       {
          mo->error = MPD_ERROR_NORESPONSE;
          DBG("ERROR @recv(), err=%s",strerror(errno));
@@ -140,6 +138,7 @@ void mpd_connect(MpdObj* mo)
           return;
       }
       mo->buflen = nbread;
+      strncpy(mo->buffer, mo->recv_buffer, mo->buflen);
       mo->buffer[mo->buflen] = '\0';
    }
    else if(err < 0)
@@ -197,7 +196,7 @@ int mpd_wait_for_answer(MpdObj *mo)
    FD_SET(mo->socket,&fds);
    if((err = select(mo->socket+1,&fds,NULL,NULL,&tv)) == 1)
    {
-      if ((nbread = recv(mo->socket, mo->buffer, MAXBUFLEN, 0)) < 0)
+      if ((nbread = recv(mo->socket, mo->recv_buffer, MAXBUFLEN, 0)) < 0)
       {
          mo->error = MPD_ERROR_NORESPONSE;
          DBG("ERROR @recv(), err=%s",strerror(errno));
@@ -210,8 +209,9 @@ int mpd_wait_for_answer(MpdObj *mo)
          return -1;
       }
 
-      DBG("Read %d bytes, buff=\"%s\"", nbread, mo->buffer);
+      DBG("Read %d bytes, buff=\"%s\"", nbread, mo->recv_buffer);
       mo->buflen = nbread;
+      strncpy(mo->buffer, mo->recv_buffer, mo->buflen);
       mo->buffer[mo->buflen] = '\0';
    }
    else if(err < 0)
@@ -286,8 +286,8 @@ void send_complex_cmd(MpdObj* mo, char* cmd, void (*parse_answer_fct)(), void *r
       DBG("Sent %d bytes",nbwri);
 
       nbread = mpd_wait_for_answer(mo);
-      /* special case for long answers with 'playlistinfo' */
-      while (nbread == MAXBUFLEN)
+      /* special case for long answers with 'playlistinfo' - hack to loop until we have received the final OK\n*/
+      while (!mo->error && ( nbread == MAXBUFLEN || 0 != strcmp(mo->buffer + strlen(mo->buffer) - 3,"OK\n")))
       {
          /* save the end of the buffer from last occurence of 'file:', and replace it with 'OK\n' */
          ptr = g_strrstr(mo->buffer, "file:");
@@ -299,8 +299,7 @@ void send_complex_cmd(MpdObj* mo, char* cmd, void (*parse_answer_fct)(), void *r
          DBG("tmp_buffer contains \"%s\"", tmp_buffer);
          DBG("buffer now contains \"%s\"", mo->buffer);
          /* parse buffer */
-         if (!mo->error)
-            (*parse_answer_fct)(mo, res);
+         (*parse_answer_fct)(mo, res);
 
          /* re-read stuff */
          nbread = mpd_wait_for_answer(mo);
@@ -378,7 +377,6 @@ void parse_one_song(MpdObj *mo, void* param)
    mpd_Song* ms = (mpd_Song*) param;
    char *eol,*ptr;
    char key[15], value[200];
-
    ms->artist = ms->album = ms->title = ms->track = NULL;
    ms->id = ms->pos = 0;
 
